! ----------------------------------------------------------------------
!        Dissertation            DI KOFLER Peter 9126050        E086 869

!        Funktionendatei: Problem 18a (2-dim. System 1. Ord.)
!                         Example 5.4, \cite{cash1}, p. 101, changed.

!        y(x) = [x**2*exp((1-x)/eps);...] (EW -1, -2, var.coeff)
!        y'(x) = [0 1; -2 -3]*y(x)/x + [0;...]
!        y(0) = [0; 0]
!        0 - 1.0
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE

!        Parameter
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   REAL(KIND = lrk), PARAMETER :: eps = 0.1D0
!  P18b hat 0.05, 0.025, 0.01
   LOGICAL, PARAMETER :: linear = .true. 
   PRIVATE
   PUBLIC :: dfcn, linear, lrk, neqn, initial, fcn, true, truelocal
!        INTEGER neqn ... Dimension des Systems als Konstante
!        SUBROUTINE initial(xstart,xend,yx0) ... Anfangsbedingungen
!        REAL FUNTION fcn ... f(x,y)
!        REAL FUNTION true ... y(x)
!        REAL FUNTION truelocal(x,xstart,yx0) ... Loesungsschar dummy

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte

   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0d0
   xend = 1.0d0
   yx0 = (/ 0.0d0, 0.0d0 /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0d0) THEN
      fcn = (/ 0.0d0, 0.0d0 /)
   ELSE
      fcn = (/ y(2)/x, (-2.0D0-8.0D0*x/eps+(x/eps)**2)*y(1)/x &
               -3.0d0*y(2)/x + 12.0D0*x*exp((1-x)/eps) /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x)
!        echte Loesung y(x)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(neqn)

   true = (/ x**2*exp((1-x)/eps), x**2*exp((1-x)/eps)*(2.0D0-x/eps) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten dummy

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   truelocal = yx0 - true(xstart) + true(x)
   RETURN
END FUNCTION truelocal

! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

   dfcn(1,:) = (/ 0.0d0, 1.0d0/x /)
   dfcn(2,:) = (/ (-2.0D0-8.0D0*x/eps+(x/eps)**2)/x, -3.0d0/x /)
   RETURN
END FUNCTION dfcn

END MODULE fun
