! ----------------------------------------------------------------------
!        Dissertation            DI KOFLER Peter 9126050        E086 869

!        Funktionendatei: Problem 22a (2-dim. System 1. Ord.)

!        y(x) = [x**2*exp(k*x);...] (Eigenwerte -1, -2)
!        y'(x) = [0 1; -2 -3]*y(x)/x + exp(k*x)*x*[0; ...]
!        y(0) = [0; 0]
!        0 - 1.0
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE

!        Parameter
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   INTEGER, PARAMETER :: k = 2
   LOGICAL, PARAMETER :: linear = .true. 
   PRIVATE
   PUBLIC :: dfcn, linear, lrk, neqn, initial, fcn, true, truelocal
!        INTEGER neqn ... Dimension des Systems als Konstante
!        SUBROUTINE initial(xstart,xend,yx0) ... Anfangsbedingungen
!        REAL FUNTION fcn ... f(x,y)
!        REAL FUNTION true ... y(x)
!        REAL FUNTION truelocal(x,xstart,yx0) ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte

   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0d0
   xend = 1.0d0
   yx0 = (/ 0.0d0, 0.0d0 /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0d0) THEN
      fcn = (/ 0.0d0, 0.0d0 /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      fcn = (/ y(2)/x, -2.0d0*y(1)/x -3.0d0*y(2)/x + &
               exp(x*k) * ((k*x+8)*k*x+12)*x /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x)
!        echte Loesung y(x)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(neqn)

   true = (/ x**2*exp(x*k), x**2*exp(x*k)*(2.0D0+x*k) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten
!        fuer lineare, konst. Koeff, l1=-1,l2=-2.

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: c1, c2, t(SIZE(yx0))
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   t = true(xstart)          ! t enthaelt die partikulaere Lsg. bei xstart
   c2 = xstart**2*(t(1)+t(2)-yx0(1)-yx0(2))
   IF (c2 == 0.0D0) THEN
        c1 = xstart*(yx0(1)-t(1))
   ELSE
        c1 = xstart*(yx0(1)-t(1))-c2/xstart
   END IF

   truelocal = (/ c1/x + c2/x/x, -c1/x -2.0D0*c2/x/x /) + true(x)
   RETURN
END FUNCTION truelocal
 
! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0d0, 1.0d0/x /)
      dfcn(2,:) = (/ -2.0d0/x, -3.0d0/x /)
   RETURN
END FUNCTION dfcn

END MODULE fun
